/**
 * VEML6075
 * Vishay UV Sensor
 */
#include "project.h"
#include "i2c_utils.h"
#include "veml6075.h"

/* VEML6075 SLAVE ADDRESS AND FUNCTION DESCRIPTION */
#define REG_UV_CONF         0x00
#define REG_Reserved01      0x01
#define REG_Reserved02      0x02
#define REG_Reserved03      0x03
#define REG_Reserved04      0x04
#define REG_Reserved05      0x05
#define REG_Reserved06      0x06
#define REG_UVA_Data        0x07
#define REG_UVD_Data        0x08
#define REG_UVB_Data        0x09
#define REG_UVCOMP1_Data    0x0A
#define REG_UVCOMP2_Data    0x0B
#define REG_ID              0x0C

// Following magic numbers are from 
// VISHAY veml6075 Application Note 84339
// Page 6 
#define UVA_A_COEF  (2.22)
#define UVA_B_COEF  (1.33)
#define UVB_C_COEF  (2.95)
#define UVB_D_COEF  (1.74)
#define UVA_sensitivity (0.93)
#define UVA_CIE_sensitivity (0.093)
#define UVB_sensitivity (2.1)
#define UVB_CIE_sensitivity (0.21)
// from page 15
#define UVA_RESPONSIVITY (0.001461)
#define UVB_RESPONSIVITY (0.002591)

static uint8_t veml6075_slave_address = 0x10 ;

void    VEML6075_setAddress(uint8_t address) 
{
    veml6075_slave_address = address ;
}

uint8_t VEML6075_getAddress(void) 
{
    return( veml6075_slave_address ) ;
}

float   VEML6075_getUVA(void) 
{
    float uva ;
    uint16_t uva_data ;
    uva_data = VEML6075_getUVAData() ;
    uva = (float)uva_data / (float)UVA_sensitivity ;
    return( uva ) ;
}

float   VEML6075_getUVA_CIE(void)
{
    float uva_cie ;
    uint16_t uva_data ;
    uva_data = VEML6075_getUVAData() ;
    uva_cie = (float)uva_data / (float)UVA_CIE_sensitivity ;
    return( uva_cie ) ;
}

float   VEML6075_getUVB(void) 
{
    float uvb ;
    uint16_t uvb_data ;
    uvb_data = VEML6075_getUVBData() ;
    uvb = (float)uvb_data / (float)UVB_sensitivity ;
    return( uvb ) ;
}

float   VEML6075_getUVB_CIE(void) 
{
    float uvb_cie ;
    uint16_t uvb_data ;
    uvb_data = VEML6075_getUVBData() ;
    uvb_cie = (float)uvb_data / (float)UVB_CIE_sensitivity ;
    return( uvb_cie ) ;
}

uint8_t VEML6075_getUVConf(void) 
{
    uint8_t data[2] ;
    uint8_t uvconf ;
    i2c_set_slave_address(veml6075_slave_address) ;
    i2c_readRegs(REG_UV_CONF, data, 2) ;
    uvconf = data[0] ;
    return( data[0] ) ;
}
   
void    VEML6075_setUVConf(uint8_t uvconf) 
{
    uint8_t data[2] ;
    data[0] = uvconf ;
    data[1] = 0 ;
    i2c_set_slave_address(veml6075_slave_address) ;
    i2c_writeRegs(REG_UV_CONF, data, 2) ; 
}

uint16_t VEML6075_getUVAData(void) 
{
    uint8_t data[2] ;
    uint16_t uvadata ;
    i2c_set_slave_address(veml6075_slave_address) ;
    i2c_readRegs(REG_UVA_Data, data, 2) ;
    uvadata = (data[1] << 8) | data[0] ;
    return( uvadata ) ;
}

uint16_t VEML6075_getUVBData(void) 
{
    uint8_t data[2] ;
    uint16_t uvbdata ;
    i2c_set_slave_address(veml6075_slave_address) ;
    i2c_readRegs(REG_UVB_Data, data, 2) ;
    uvbdata = (data[1] << 8) | data[0] ;
    return( uvbdata ) ;    
}

uint16_t VEML6075_getUVDData(void) 
{
    uint8_t data[2] ;
    uint16_t uvddata ;
    i2c_set_slave_address(veml6075_slave_address) ;
    i2c_readRegs(REG_UVD_Data, data, 2) ;
    uvddata = (data[1] << 8) | data[0] ;
    return( uvddata ) ;
}

uint16_t VEML6075_getUVCOMP1Data(void) 
{
    uint8_t data[2] ;
    uint16_t uvcomp1 ;
    i2c_set_slave_address(veml6075_slave_address) ;
    i2c_readRegs(REG_UVCOMP1_Data, data, 2) ;    
    uvcomp1 = (data[1] << 8) | data[0] ;
    return( uvcomp1 ) ;
}

uint16_t VEML6075_getUVCOMP2Data(void) 
{
    uint8_t data[2] ;
    uint16_t uvcomp2 ;
    i2c_set_slave_address(veml6075_slave_address) ;
    i2c_readRegs(REG_UVCOMP2_Data, data, 2) ;    
    uvcomp2 = (data[1] << 8) | data[0] ;
    return( uvcomp2 ) ;
}

float   VEML6075_UVI(void) 
{
    float uvi = 0.0 ;
    uint16_t uvacomp = 0.0 ;
    uint16_t uvbcomp = 0.0 ;
    uint16_t uva, uvb, uvd, uvcomp1, uvcomp2 ;
    uva = VEML6075_getUVAData() ;
    uvb = VEML6075_getUVBData() ;
    uvd = VEML6075_getUVDData() ;
    uvcomp1 = VEML6075_getUVCOMP1Data() ;
    uvcomp2 = VEML6075_getUVCOMP2Data() ;
    uvacomp = (uva - uvd) - UVA_A_COEF * (uvcomp1 - uvd) - UVA_B_COEF * (uvcomp2 - uvd) ;
    uvbcomp = (uvb - uvd) - UVB_C_COEF * (uvcomp1 - uvd) - UVB_D_COEF * (uvcomp2 - uvd) ;
//    uvi = (((float)uvbcomp * (float)UVB_RESPONSIVITY) + ((float)uvacomp * (float)UVA_RESPONSIVITY)) / 2.0 * 15 ; // For demo
    uvi = (((float)uvbcomp * (float)UVB_RESPONSIVITY) + ((float)uvacomp * (float)UVA_RESPONSIVITY)) / 2.0 ;
    return( uvi ) ;    
}

uint16_t VEML6075_getID(void) 
{
    uint16_t id ;
    uint8_t data[2] ;
    i2c_set_slave_address(veml6075_slave_address) ;
    i2c_readRegs(REG_ID, data, 2) ;        
    id = (data[1] << 8) | data[0] ;
    return( id ) ;
}